<?php
namespace Teknisa\PLA\Modules\Prato\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Util\QuerySelector;

class Pla00470EmbalagemTransportada extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "contembalag";
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $entityManager) {
    	$this->crudService = $crudService;
    	$this->parameterBag = $parameterBag;
    	$this->entityManager = $entityManager;
    }

    public function salvaEmbalagemTransportada(DTO\Request $request, DTO\Response $response) {
    	try {
            $row = $request->getRow();
            if ($row['__is_new']) {
                $this->verificaEmbalagemJaCadastrada($row);
                $row['NRCONTEMBALAG'] = $this->retornaProximoCodigo();
                $response->addDataset(new DataSet('NRCONTEMBALAG', array($row['NRCONTEMBALAG'])));
            }
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
        }
    }

    private function retornaProximoCodigo() {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
		$queryOracle = <<<SQL
		SELECT NVL(MAX(NRCONTEMBALAG), 0) + 1 AS NRCONTEMBALAG FROM CONTEMBALAG WHERE NRORG = :NRORG
SQL;
        $queryPostgre = <<<SQL
        SELECT COALESCE(MAX(NRCONTEMBALAG), 0) + 1 AS "NRCONTEMBALAG" FROM CONTEMBALAG WHERE NRORG = :NRORG
SQL;
		$params = array("NRORG" => $this->crudService->getEnvironment()->getNrOrg());
		$arrayQueries = array (
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        );
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        $result = $connection->fetchAll($query, $params);
        return $result[0]['NRCONTEMBALAG'];
	}

	private function verificaEmbalagemJaCadastrada($row) {
		$sql = 'SELECT NRCONTEMBALAG AS "NRCONTEMBALAG" FROM CONTEMBALAG WHERE NRORG = :NRORG AND CDEMBALAGEM = :CDEMBALAGEM AND CDCONTAINER = :CDCONTAINER';
		$params = array(
			"NRORG" => $this->crudService->getEnvironment()->getNrOrg(),
			"CDCONTAINER" => $row["CDCONTAINER"],
			"CDEMBALAGEM" => $row["CDEMBALAGEM"],
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        if (sizeof($result) > 0) {
        	throw new \Exception("Impossível adicionar. Registro já existe.");
        }
	}

}